@@ -20,13 +20,15 @@
 //! }
 //! ```
 
+#![allow(clippy::manual_range_contains)]
+
 use crate::{buffer::DataSource, decoder::Decoder};
 use rg3d_core::visitor::{Visit, VisitResult, Visitor};
 use std::path::Path;
 use std::{path::PathBuf, time::Duration};
 
 /// Generic sound buffer that contains decoded samples and allows random access.
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct GenericBuffer {
     /// Interleaved decoded samples (mono sounds: L..., stereo sounds: LR...)
     /// For streaming buffers it contains only small part of decoded data
@@ -37,17 +39,6 @@ pub struct GenericBuffer {
     pub(in crate) external_source_path: PathBuf,
 }
 
-impl Default for GenericBuffer {
-    fn default() -> Self {
-        Self {
-            samples: Vec::new(),
-            channel_count: 0,
-            sample_rate: 0,
-            external_source_path: Default::default(),
-        }
-    }
-}
-
 impl Visit for GenericBuffer {
     fn visit(&mut self, name: &str, visitor: &mut Visitor) -> VisitResult {
         visitor.enter_region(name)?;
@@ -64,6 +55,9 @@ impl GenericBuffer {
     ///
     /// # Notes
     ///
+    /// `DataSource::RawStreaming` is **not** supported with generic buffers, use streaming buffer
+    /// instead!
+    ///
     /// Data source with raw samples must have sample count multiple of channel count, otherwise this
     /// function will return `Err`.
     pub fn new(source: DataSource) -> Result<Self, DataSource> {
@@ -73,7 +67,7 @@ impl GenericBuffer {
                 channel_count,
                 samples,
             } => {
-                if samples.len() % channel_count != 0 {
+                if channel_count < 1 || channel_count > 2 || samples.len() % channel_count != 0 {
                     Err(DataSource::Raw {
                         sample_rate,
                         channel_count,
@@ -88,6 +82,7 @@ impl GenericBuffer {
                     })
                 }
             }
+            DataSource::RawStreaming(_) => Err(source),
             _ => {
                 let external_source_path = if let DataSource::File { path, .. } = &source {
                     path.clone()
@@ -95,7 +90,28 @@ impl GenericBuffer {
                     Default::default()
                 };
 
+                // Store cursor to handle errors.
+                let (is_memory, external_cursor) = if let DataSource::Memory(cursor) = &source {
+                    (true, cursor.clone())
+                } else {
+                    (false, Default::default())
+                };
+
                 let decoder = Decoder::new(source)?;
+                if decoder.get_channel_count() < 1 || decoder.get_channel_count() > 2 {
+                    if is_memory {
+                        return Err(DataSource::Memory(external_cursor));
+                    } else {
+                        // There is not much we can do here: if the user supplied DataSource::File,
+                        // they probably do not want us to re-read the file again in
+                        // DataSource::from_file.
+                        return Err(DataSource::Raw {
+                            sample_rate: decoder.get_sample_rate(),
+                            channel_count: decoder.get_channel_count(),
+                            samples: vec![],
+                        });
+                    }
+                }
 
                 Ok(Self {
                     sample_rate: decoder.get_sample_rate(),
@@ -156,9 +172,4 @@ impl GenericBuffer {
             (self.samples.len() / (self.channel_count * self.sample_rate)) as f64,
         )
     }
-
-    #[inline]
-    pub(in crate) fn index_of_last_sample(&self) -> usize {
-        self.samples.len() - self.channel_count
-    }
 }
