@@ -26,6 +26,7 @@
 //! Streaming buffer cannot be shared across multiple source. On attempt to create a source with a streaming
 //! buffer that already in use you'll get error.
 
+use crate::buffer::RawStreamingDataSource;
 use crate::{
     buffer::{generic::GenericBuffer, DataSource},
     decoder::Decoder,
@@ -36,37 +37,112 @@ use std::ops::{Deref, DerefMut};
 use std::time::Duration;
 
 /// Streaming buffer for long sounds. Does not support random access.
-#[derive(Debug)]
+#[derive(Debug, Default)]
 pub struct StreamingBuffer {
     pub(in crate) generic: GenericBuffer,
     /// Count of sources that share this buffer, it is important to keep only one
     /// user of streaming buffer, because streaming buffer does not allow random
     /// access.
     pub(in crate) use_count: usize,
-    decoder: Decoder,
+    streaming_source: StreamingSource,
 }
 
-impl Default for StreamingBuffer {
+#[derive(Debug)]
+enum StreamingSource {
+    Null,
+    Decoder(Decoder),
+    Raw(Box<dyn RawStreamingDataSource>),
+}
+
+impl Default for StreamingSource {
     fn default() -> Self {
-        Self {
-            generic: Default::default(),
-            decoder: Decoder::Null,
-            use_count: 0,
-        }
+        Self::Null
     }
 }
 
-#[inline]
-fn read_samples(buffer: &mut Vec<f32>, decoder: &mut Decoder, count: usize) -> usize {
-    buffer.clear();
-    for _ in 0..count {
-        if let Some(sample) = decoder.next() {
-            buffer.push(sample)
-        } else {
-            break;
+impl StreamingSource {
+    #[inline]
+    fn new(data_source: DataSource) -> Result<Self, DataSource> {
+        match data_source {
+            DataSource::File { .. } | DataSource::Memory(_) => {
+                Ok(Self::Decoder(Decoder::new(data_source)?))
+            }
+            DataSource::RawStreaming(raw) => Ok(Self::Raw(raw)),
+            // It makes no sense to stream raw data which is already loaded into memory.
+            _ => Err(data_source),
         }
     }
-    buffer.len()
+
+    #[inline]
+    fn sample_rate(&self) -> usize {
+        match self {
+            StreamingSource::Decoder(decoder) => decoder.get_sample_rate(),
+            StreamingSource::Raw(raw) => raw.sample_rate(),
+            StreamingSource::Null => 0,
+        }
+    }
+
+    #[inline]
+    fn channel_count(&self) -> usize {
+        match self {
+            StreamingSource::Decoder(decoder) => decoder.get_channel_count(),
+            StreamingSource::Raw(raw) => raw.channel_count(),
+            StreamingSource::Null => 0,
+        }
+    }
+
+    fn duration(&self) -> Option<Duration> {
+        match self {
+            StreamingSource::Null => None,
+            StreamingSource::Decoder(decoder) => decoder.duration(),
+            StreamingSource::Raw(raw) => raw.duration(),
+        }
+    }
+
+    fn rewind(&mut self) -> Result<(), SoundError> {
+        match self {
+            StreamingSource::Null => Ok(()),
+            StreamingSource::Decoder(decoder) => decoder.rewind(),
+            StreamingSource::Raw(raw) => raw.rewind(),
+        }
+    }
+
+    fn time_seek(&mut self, location: Duration) {
+        match self {
+            StreamingSource::Null => {}
+            StreamingSource::Decoder(decoder) => decoder.time_seek(location),
+            StreamingSource::Raw(raw) => raw.time_seek(location),
+        }
+    }
+
+    #[inline]
+    fn read_next_samples_block_into(&mut self, buffer: &mut Vec<f32>) -> usize {
+        buffer.clear();
+        let count = StreamingBuffer::STREAM_SAMPLE_COUNT * self.channel_count();
+        match self {
+            StreamingSource::Decoder(decoder) => {
+                for _ in 0..count {
+                    if let Some(sample) = decoder.next() {
+                        buffer.push(sample)
+                    } else {
+                        break;
+                    }
+                }
+            }
+            StreamingSource::Raw(raw_streaming) => {
+                for _ in 0..count {
+                    if let Some(sample) = raw_streaming.next() {
+                        buffer.push(sample)
+                    } else {
+                        break;
+                    }
+                }
+            }
+            StreamingSource::Null => (),
+        }
+
+        buffer.len()
+    }
 }
 
 impl StreamingBuffer {
@@ -82,61 +158,50 @@ impl StreamingBuffer {
     /// This function will return Err if data source is `Raw`. It makes no sense to stream raw data which
     /// is already loaded into memory. Use Generic source instead!
     pub fn new(source: DataSource) -> Result<Self, DataSource> {
-        if let DataSource::Raw { .. } = source {
-            return Err(source);
-        };
-
         let external_source_path = if let DataSource::File { path, .. } = &source {
             path.clone()
         } else {
             Default::default()
         };
 
-        let mut decoder = Decoder::new(source)?;
+        let mut streaming_source = StreamingSource::new(source)?;
 
         let mut samples = Vec::new();
-        let channel_count = decoder.get_channel_count();
-        read_samples(
-            &mut samples,
-            &mut decoder,
-            Self::STREAM_SAMPLE_COUNT * channel_count,
-        );
+        let channel_count = streaming_source.channel_count();
+        streaming_source.read_next_samples_block_into(&mut samples);
         debug_assert_eq!(samples.len() % channel_count, 0);
 
         Ok(Self {
             generic: GenericBuffer {
                 samples,
-                sample_rate: decoder.get_sample_rate(),
-                channel_count: decoder.get_channel_count(),
+                sample_rate: streaming_source.sample_rate(),
+                channel_count: streaming_source.channel_count(),
                 external_source_path,
             },
             use_count: 0,
-            decoder,
+            streaming_source,
         })
     }
 
     /// Returns total duration of data. Can be `None` if internal decoder does not supports seeking.
     pub fn duration(&self) -> Option<Duration> {
-        self.decoder.duration()
+        self.streaming_source.duration()
     }
 
     #[inline]
     pub(in crate) fn read_next_block(&mut self) {
-        read_samples(
-            &mut self.generic.samples,
-            &mut self.decoder,
-            self.generic.channel_count * Self::STREAM_SAMPLE_COUNT,
-        );
+        self.streaming_source
+            .read_next_samples_block_into(&mut self.generic.samples);
     }
 
     #[inline]
     pub(in crate) fn rewind(&mut self) -> Result<(), SoundError> {
-        self.decoder.rewind()
+        self.streaming_source.rewind()
     }
 
     #[inline]
     pub(in crate) fn time_seek(&mut self, location: Duration) {
-        self.decoder.time_seek(location);
+        self.streaming_source.time_seek(location);
     }
 }
 
