@@ -8,7 +8,7 @@
 //! for inventory and so on. With this approach of multiple contexts it is very easy to manage such scenes:
 //! for example your main menu have a complex scene with some sounds and you decide to load a game level -
 //! once the level is loaded you just set master gain of main menu context and it will no longer produce any
-//! sounds, only your level will do.  
+//! sounds, only your level will do.
 
 use crate::pool::Ticket;
 use crate::{
@@ -253,13 +253,10 @@ impl State {
         let last_time = rg3d_core::instant::Instant::now();
 
         if !self.paused {
-            for i in 0..self.sources.get_capacity() {
-                if let Some(source) = self.sources.at(i) {
-                    if source.is_play_once() && source.status() == Status::Stopped {
-                        self.sources.free(self.sources.handle_from_index(i));
-                    }
-                }
-            }
+            self.sources.retain(|source| {
+                let done = source.is_play_once() && source.status() == Status::Stopped;
+                !done
+            });
 
             for source in self
                 .sources
