@@ -32,22 +32,32 @@ use crate::{
     listener::Listener,
     source::{generic::GenericSource, SoundSource},
 };
-use rg3d_core::algebra::Vector3;
-use rg3d_core::visitor::{Visit, VisitResult, Visitor};
+use rg3d_core::{
+    algebra::Vector3,
+    inspect::{Inspect, PropertyInfo},
+    visitor::{Visit, VisitResult, Visitor},
+};
 use std::ops::{Deref, DerefMut};
 
 /// See module docs.
-#[derive(Debug, Clone)]
+#[derive(Debug, Clone, Inspect)]
 pub struct SpatialSource {
     pub(in crate) generic: GenericSource,
+    #[inspect(min_value = 0.0, step = 0.05)]
     radius: f32,
     position: Vector3<f32>,
+    #[inspect(min_value = 0.0, step = 0.05)]
     max_distance: f32,
+    #[inspect(min_value = 0.0, step = 0.05)]
     rolloff_factor: f32,
     // Some data that needed for iterative overlap-save convolution.
+    #[inspect(skip)]
     pub(in crate) prev_left_samples: Vec<f32>,
+    #[inspect(skip)]
     pub(in crate) prev_right_samples: Vec<f32>,
+    #[inspect(skip)]
     pub(in crate) prev_sampling_vector: Vector3<f32>,
+    #[inspect(skip)]
     pub(in crate) prev_distance_gain: Option<f32>,
 }
 
